'From Squeak3.6alpha of ''17 March 2003'' [latest update: #5325] on 18 July 2003 at 9:59:38 am'!
"Change Set:		OSProcessV3-0-6-dtl
Date:			18 July 2003
Author:			David T. Lewis

This is in incremental update to OSProcess 3.0.5.
This update includes an important bug fix for OSPP 3.0.5. All other versions
are OK, including the current SAR on Squeak Map.

Changes since 3.0.5:

- Fixed an error in the Unix signal handler (due to a typo by yours truly in
the last patch release, version 3.0.5). Without this fix, it is possible for the VM
to exit without any error message or stack trace (this is the 'normal' behavior
of the VM on receipt of a SIGPIPE signal). I am not aware of this happening in
practice for any current OSProcess users, but it did happen to me in the course
of reimplement some new CommandShell pipeline methods, so I consider this an
important fix.

- Made various fixes to maintain compatibility with VMMaker code generation.

- Fixed OSProcessPlugin class>>installedModule so it works even if the plugin
classes were loaded without VMMaker (in which case InterpreterPlugin does
not exist in the image).

- Made a minor change to follow PackageInfo naming conventions for
StandardFileStream>>fileID.

- Fixed undeclared references (classes in CommandShell and IOHandle).

- Update version numbers for OSProcess and OSProcessPlugin.
"!


!OSProcess class methodsFor: 'version testing' stamp: 'dtl 7/13/2003 14:57'!
versionInformation

	"OSProcess versionInformation"

	| osppVersion |
	osppVersion _ (Smalltalk hasClassNamed: #OSProcessPlugin)
		ifTrue:
			[(Smalltalk at: #OSProcessPlugin) installedModule]
		ifFalse:
			['(not installed in this image)'].
	^ Array
		with: (self name, ' version ', self versionString)
		with: ((Smalltalk hasClassNamed: #CommandShell)
				ifTrue:
					[((Smalltalk at: #CommandShell) respondsTo: #versionString)
						ifTrue:
							['CommandShell version ', (Smalltalk at: #CommandShell) versionString]
						ifFalse:
							['CommandShell installed (old version, no versionString)']]
				ifFalse:
					['CommandShell is not installed'])
		with:  osppVersion
! !

!OSProcess class methodsFor: 'version testing' stamp: 'dtl 7/12/2003 11:09'!
versionString

	"OSProcess versionString"

	^'3.0.6'! !


!ExternalUnixOSProcess class methodsFor: 'instance creation' stamp: 'dtl 7/12/2003 11:38'!
command: aCommandString

	"ExternalUnixOSProcess command: 'ls -l /etc'"

	^ self forkAndExec: self defaultShellPath
		arguments: (Array with: '-c' with: aCommandString)
		environment: nil! !


!OSProcessAccessor methodsFor: 'private - IOHandle' stamp: 'dtl 7/12/2003 12:17'!
ioAccessorFromSQFile: aByteArray
	"Answer an object which represents an IO channel. If IOHandle is present in
	this image, use it; otherwise just answer aByteArray."

	UseIOHandle
		ifTrue: [^ (Smalltalk at: #IOHandle) newFromSqFileStruct: aByteArray]
		ifFalse: [^ aByteArray]! !


!OSProcessPlugin methodsFor: 'primitives - OS process access' stamp: 'dtl 7/14/2003 19:44'!
primitiveGetSession
	"Answer the unique session identifier for this Smalltalk instance running in this
	OS process. The C integer value is coerced into a Smalltalk ByteArray to preserve
	the full range of possible values."

	| sessionOop sessionByteArrayPointer thisSessionID |
	self export: true.
	self var: 'sessionByteArrayPointer' declareC: 'unsigned char *sessionByteArrayPointer'.
	self var: 'copyByteArrayPointer' declareC: 'unsigned char *copyByteArrayPointer'.
	sessionOop _ interpreterProxy instantiateClass: interpreterProxy classByteArray indexableSize: self sizeOfInt.
	sessionByteArrayPointer _ interpreterProxy arrayValueOf: sessionOop.
	thisSessionID _ self getThisSessionIdentifier.
	thisSessionID isNil ifTrue:
		[^ interpreterProxy primitiveFail].	"The session ID is not permitted to be zero"
	self copyBytesFrom: (self cCode: '(unsigned char *)&thisSessionID' inSmalltalk: [thisSessionID])
		to: sessionByteArrayPointer
		length: self sizeOfInt.
	interpreterProxy pop: 1; push: sessionOop
! !

!OSProcessPlugin methodsFor: 'primitives - machine specific' stamp: 'dtl 7/14/2003 06:32'!
primitiveSizeOfInt
	"Size in bytes of an integer, for this C compiler on this machine."

	self export: true.
	interpreterProxy pop: 1; pushInteger: self sizeOfInt! !

!OSProcessPlugin methodsFor: 'primitives - machine specific' stamp: 'dtl 7/14/2003 06:33'!
primitiveSizeOfPointer
	"Size in bytes of a void pointer, for this C compiler on this machine."

	self export: true.
	interpreterProxy pop: 1; pushInteger: self sizeOfPointer! !

!OSProcessPlugin methodsFor: 'version identification' stamp: 'dtl 7/14/2003 05:48'!
versionString
	"Answer a string containing the version string for this plugin. Handle MNU
	errors, which can occur if class InterpreterPlugin has been removed from
	the system.

	Important: When this method is changed, the class side method must also be
	changed to match."

	| version |
	self returnTypeC: 'char *'.
	self var: 'version' declareC: 'static char version[]= "3.0.6"'.
	^ self cCode: 'version' inSmalltalk: [self class versionString]
! !

!OSProcessPlugin methodsFor: 'private - conversion' stamp: 'dtl 7/14/2003 19:45'!
copyBytesFrom: charArray1 to: charArray2 length: len

	"| chars |
	chars _ 'abcd'.
	self new copyBytesFrom: 'wxyz' to: chars length: 4.
	chars"

	| p1 p2 idx |
	self var: 'charArray1' type: 'unsigned char *'.
	self var: 'charArray2' type: 'unsigned char *'.
	self var: 'p1' declareC: 'unsigned char *p1'.
	self var: 'p2' declareC: 'unsigned char *p2'.
	self cCode: 'p1 = charArray1' inSmalltalk: [p1 _ 1].
	self cCode: 'p2 = charArray2' inSmalltalk: [p2 _ 1].
	idx _ 0.
	[idx < len] whileTrue:
		[self cCode: '*p2 = *p1'
			inSmalltalk: [charArray2 at: p2 put: (charArray1 at: p1)].
		p1 _ p1 + 1.
		p2 _ p2 + 1.
		idx _ idx + 1]
! !

!OSProcessPlugin methodsFor: 'private - conversion' stamp: 'dtl 7/14/2003 06:31'!
sizeOfInt
	"Size in bytes of an integer, for this C compiler on this machine."

	^ self cCode: 'sizeof(int)' inSmalltalk: [4]
! !

!OSProcessPlugin methodsFor: 'private - conversion' stamp: 'dtl 7/14/2003 06:32'!
sizeOfPointer
	"Size in bytes of a void pointer, for this C compiler on this machine."

	^ self cCode: 'sizeof(void *)' inSmalltalk: [4].
! !


!OSProcessPlugin class methodsFor: 'version identification' stamp: 'dtl 7/14/2003 05:46'!
versionString
	"Version numbering is independent of OSProcess class>>versionString, and should be
	changed only when the plugin changes. Version numbers will begin arbitrarily
	with 2.1 as of OSProcess 2.7. See the instance side version of this method for the
	implementation.

	Important: When this method is changed, the instance side method must also be
	changed to match."

	"OSProcessPlugin versionString"

	^ '3.0.6'! !


!UnixOSProcessPlugin methodsFor: 'primitives - OS process access' stamp: 'dtl 7/14/2003 19:43'!
primitiveReapChildProcess
	"Clean up after the death of a child, and answer an array with the pid and
	the exit status of the child process. Answer nil if the pidToHandle does not exist."

	| pidToHandle pidResult exitStatus resultArray arrayPtr |
	self export: true.
	self var: 'arrayPtr' declareC: 'int *arrayPtr'.
	exitStatus _ 0.	"Force C code translator to declare the variable"
	pidToHandle _ interpreterProxy stackIntegerValue: 0.
	pidResult _ self cCode: 'waitpid ( pidToHandle, &exitStatus, WNOHANG )'
					inSmalltalk: [ exitStatus _ -1 ].
	pidResult <= 0
		ifTrue: [ interpreterProxy pop: 2; push: interpreterProxy nilObject ]
		ifFalse: [ "Answer an array with pid and result status "
			resultArray _ interpreterProxy
							instantiateClass: interpreterProxy classArray
							indexableSize: 2.
			arrayPtr _ interpreterProxy firstIndexableField: resultArray.
			self cCode: 'arrayPtr[0] = integerObjectOf(pidResult)'
				inSmalltalk: [resultArray at: 1 put: pidResult].
			self cCode: 'arrayPtr[1] = integerObjectOf(exitStatus)'
				inSmalltalk: [resultArray at: 2 put: exitStatus].
			interpreterProxy pop: 2; push: resultArray ]
! !

!UnixOSProcessPlugin methodsFor: 'private - signum' stamp: 'dtl 7/12/2003 11:07'!
sigIgnoreNumber
	"Ignore action for a signal"

	self returnTypeC: 'void *'.
	^ self cCode: 'SIG_IGN' inSmalltalk: [1]
! !


!UnixProcess methodsFor: 'external command processing' stamp: 'dtl 7/12/2003 11:51'!
command: aCommandString input: aStreamOrString
	"Run a command in a shell process. Similar to the system(3) call in the standard C library,
	except that aCommandString runs asynchronously in a child process."

	"OSProcess thisOSProcess
		command: 'cat'
		input: 'this is some test data'"

	"OSProcess thisOSProcess
		command: 'cat'
		input: (ReadStream on: 'this is some test data')"

	| proc |
	(Smalltalk hasClassNamed: #PipeableOSProcess)
		ifTrue:
			[proc _ (Smalltalk at: #PipeableOSProcess) command: aCommandString.
			proc ifNil: [^ nil].
			proc nextPutAll: aStreamOrString contents.
			proc pipeToInput close.
			^ proc]
		ifFalse:
			[self notify: 'the #command:input: method requires CommandShell, using #command: instead'.
			^ self command: aStreamOrString contents]

! !

!UnixProcess methodsFor: 'external command processing' stamp: 'dtl 7/12/2003 12:06'!
waitForCommandOutput: aCommandString 
	"Run a command in a shell process. Similar to the system(3) call in the 
	standard C library. The active Smalltalk process waits for completion of
	the external command process."

	"OSProcess thisOSProcess waitForCommandOutput: 'echo sleeping...; sleep 1; echo I just slept for one second'"
	"OSProcess thisOSProcess waitForCommandOutput: 'ThisIsABogusCommand'"
	"OSProcess thisOSProcess waitForCommandOutput: '/bin/ls -l /etc /bin'"
	"OSProcess thisOSProcess waitForCommandOutput: 'echo Hello world!!; ls /NOSUCHFILE'"

	(Smalltalk hasClassNamed: #PipeableOSProcess)
		ifTrue:
			[^ ((Smalltalk at: #PipeableOSProcess) command: aCommandString) output]
		ifFalse:
			[self notify: 'the #waitForCommandOutput: method requires CommandShell'.
			^ '']
! !

!UnixProcess methodsFor: 'external command processing' stamp: 'dtl 7/12/2003 12:08'!
waitForCommandOutputArray: aCommandString
	"Run a command in a shell process. Similar to the system(3) call in the standard C library.
	The active Smalltalk process waits for completion of the external command process."

	"OSProcess thisOSProcess waitForCommandOutputArray: 'echo Hello world!!; ls /NOSUCHFILE'"

	| proc |
	(Smalltalk hasClassNamed: #PipeableOSProcess)
		ifTrue:
			[proc _ (Smalltalk at: #PipeableOSProcess) command: aCommandString.
			^ Array
				with: proc output
				with: proc errorUpToEnd
				with: proc processProxy exitStatus]
		ifFalse:
			[self notify: 'the #waitForCommandOutputArray: method requires CommandShell'.
			^ Array with: '' with: '' with: nil]
! !


!UnixProcess class methodsFor: 'utility' stamp: 'dtl 7/13/2003 14:47'!
startSwiki: aSwiki onPort: num loggingTo: aFileName
	"Start a swiki in a headless Squeak image."

	"UnixProcess startSwiki: 'myswiki' onPort: 8081 loggingTo: 'log.txt'"

	| proc |
	(Smalltalk hasClassNamed: #SwikiAction)
		ifTrue:
			[proc _ self forkSqueakAndDo:
				[(Smalltalk at: #SwikiAction) new restore: 'myswiki'.
				(Smalltalk at: #PWS) serveOnPort: num loggingTo: aFileName.
				UnixProcess decapitate].
			proc ifNil: [self noAccessorAvailable].
			^ proc]
		ifFalse:
		[self notify: 'PWS not installed in this image']
! !

!UnixProcess class methodsFor: 'unit tests' stamp: 'dtl 7/12/2003 12:13'!
runTests
	"Run a few tests to see if things are working correctly on Unix/Linux. 
	Output is on stdout, stderr, and the Squeak Transcript. One of the tests 
	requires input from stdin, so Squeak should be run from a shell command
	line and not as a background process.

	Warning: This test will crash your VM if your are using the -xshm command
	line option. For reasons which I do not quite understand, the X shared
	memory segment becomes invalid when the Squeak VM which initially
	opened the shared memory exits. The remaining Squeak children will crash
	when then then next try to update the display.

	Note: If you see 'select: Bad file descriptor' messages on your console standard
	output, these are occuring while running headless in the decapitate/recapitate
	tests.

	Important: Prior to evaluating this method, please type one line of text
	followed by a <cr> on the terminal standard input. This provides the
	input for the stdin test. Failing to provide this input prior to evaluating
	the tests will cause one of the test cases to fail."

	"UnixProcess runTests"

	| this s p failures result a |
	failures _ 0.
	Transcript show: 'Begin OSProcess tests'; cr.
	Transcript show: 'Test for working ProcessAccessor ... '.
	this _ OSProcess thisOSProcess.
	(this pid isKindOf: Integer)
		ifTrue: [Transcript show: 'OK'; cr]
		ifFalse: [Transcript show: 'NFG'; cr. failures _ failures + 1].
	Transcript show: 'Echo one line of text previously entered from stdin ... '.
	s _ OSProcess readFromStdIn.
	s size > 0
		ifTrue: [Transcript show: 'OK'; cr; show: s; cr]
		ifFalse: [Transcript show: 'NFG'; cr. failures _ failures + 1].
	Transcript show: 'Message to stdout ... '.
	(OSProcess helloWorld isKindOf: AttachableFileStream)
		ifTrue: [Transcript show: 'OK'; cr; show: s; cr]
		ifFalse: [Transcript show: 'NFG'; cr. failures _ failures + 1].
	Transcript show: 'Message to stderr ... '.
	(OSProcess helloStdErr isKindOf: AttachableFileStream)
		ifTrue: [Transcript show: 'OK'; cr; show: s; cr]
		ifFalse: [Transcript show: 'NFG'; cr. failures _ failures + 1].
	Transcript show: 'UnixProcess cataFile ... '.
	p _ UnixProcess catAFile.
	(Delay forSeconds: 1) wait.
	p exitStatus == 0
		ifTrue: [Transcript show: 'OK'; cr]
		ifFalse: [Transcript show: 'NFG'; cr. failures _ failures + 1].
	Transcript show: 'UnixProcess testEnvSet ... '.
	p _ UnixProcess testEnvSet.
	(Delay forSeconds: 1) wait.
	p exitStatus == 0
		ifTrue: [Transcript show: 'OK'; cr]
		ifFalse: [Transcript show: 'NFG'; cr. failures _ failures + 1].
	Transcript show: 'UnixProcess catFromFileToFiles ... '.
	OSProcess command: 'rm /tmp/deleteMe.out'.
	OSProcess command: 'rm /tmp/deleteMe.err'.
	p _ UnixProcess catFromFileToFiles.
	(Delay forSeconds: 1) wait.
	p exitStatus == 0
		ifTrue: [Transcript show: 'OK'; cr]
		ifFalse: [Transcript show: 'NFG'; cr. failures _ failures + 1].
	Transcript show: 'UnixProcess testRunCommand ... '.
	OSProcess command: 'rm /tmp/deleteMe.out'.
	OSProcess command: 'rm /tmp/deleteMe.err'.
	p _ UnixProcess testRunCommand.
	(Delay forSeconds: 1) wait.
	p exitStatus == 256
		ifTrue: [Transcript show: 'OK'; cr]
		ifFalse: [Transcript show: 'NFG'; cr. failures _ failures + 1].
	Transcript show: 'UnixProcess testPipe ... '.
	UnixProcess testPipe = 'this is some text to write into the pipe'
		ifTrue: [Transcript show: 'OK'; cr]
		ifFalse: [Transcript show: 'NFG'; cr. failures _ failures + 1].
	Transcript show: 'UnixProcess testPipeLine ... '.
	UnixProcess testPipeLine = 'This is the text to write out through one pipe, copy through an external cat command, and then read back in through another pipe.'
		ifTrue: [Transcript show: 'OK'; cr]
		ifFalse: [Transcript show: 'NFG'; cr. failures _ failures + 1].
	Transcript show: 'UnixProcess spawnTenHeadlessChildren ... '.
	p _ UnixProcess spawnTenHeadlessChildren.
	(p size == 10 and: [(p select: [:e | (e runState == #running) | (e exitStatus == 0)]) size == 10])
		ifTrue: [Transcript show: 'OK'; cr]
		ifFalse: [Transcript show: 'NFG'; cr. failures _ failures + 1].
	Transcript show: 'UnixProcess cloneSqueak ... '.
	p _ UnixProcess cloneSqueak.
	(p isKindOf: UnixProcess)
		ifTrue: 
			[(Delay forSeconds: 1) wait.
			Smalltalk snapshot: false andQuit: true].
	(Delay forSeconds: 5) wait.
	p exitStatus == 0
		ifTrue: [Transcript show: 'OK'; cr]
		ifFalse: [Transcript show: 'NFG'; cr. failures _ failures + 1].
	Transcript show: 'UnixProcess squeakSqueak ... '.
	p _ UnixProcess squeakSqueak.
	(Delay forSeconds: 5) wait.
	OSProcess thisOSProcess sigkill: p.
	(Delay forSeconds: 1) wait.
	p exitStatus == 9
		ifTrue: [Transcript show: 'OK'; cr]
		ifFalse: [Transcript show: 'NFG'; cr. failures _ failures + 1].
	Transcript show: 'UnixProcess restartVirtualMachine ... '.
	p _ UnixProcess restartVirtualMachine.
	p pid == OSProcess thisOSProcess pid
		ifTrue: [Transcript show: 'OK'; cr]
		ifFalse: [Transcript show: 'NFG'; cr. failures _ failures + 1].
	Transcript show: 'UnixProcess command: ''cat'' input: ''this is some test data'' ... '.
	p _ (OSProcess thisOSProcess
		command: 'cat'
		input: 'this is some test data').
	(Delay forSeconds: 1) wait.
	p upToEnd = 'this is some test data'
		ifTrue: [Transcript show: 'OK'; cr]
		ifFalse: [Transcript show: 'NFG'; cr. failures _ failures + 1].
	Transcript show: 'UnixProcess waitForCommandOutput: ''echo sleeping...; sleep 1; echo I just slept for one second'' ... '.
	('*I just slept for one second*' match:
		(OSProcess thisOSProcess
			waitForCommandOutput: 'echo sleeping...; sleep 1; echo I just slept for one second'))
		ifTrue: [Transcript show: 'OK'; cr]
		ifFalse: [Transcript show: 'NFG'; cr. failures _ failures + 1].
	Transcript show: 'Unix command pipeline with output and error returned in an array ... '.
	(Smalltalk hasClassNamed: #PipeableOSProcess)
		ifTrue:
			[a _ (((Smalltalk at: #PipeableOSProcess) command: 'echo this is a test; BOGUS') | 'cut -c11-14') outputAndError.
			(((a isKindOf: Array)
				and: ['test*' match: (a at: 1)])
					and: ['*BOGUS*' match: (a at: 2)])
						ifTrue: [Transcript show: 'OK'; cr]
						ifFalse: [Transcript show: 'NFG'; cr. failures _ failures + 1]]
		ifFalse:
			[Transcript show: 'skipping PipeableOSProcess test (requires CommandShell)'; cr].

	Transcript show: 'UnixProcess decapitate and recapatiate five times'; cr.
	5 timesRepeat:
		[UnixProcess decapitate.
		UnixProcess recapitate].

	failures == 1
		ifTrue: [result _ 'OSProcess tests completed with ', failures printString, ' failure']
		ifFalse: [result _ 'OSProcess tests completed with ', failures printString, ' failures'].
	Transcript show: result; cr.

	^ result
! !


!StandardFileStream reorganize!
('open/close' close closed ensureOpen open open:forWrite: openReadOnly reopen)
('properties-setting' asHtml ascii binary getFileType insertLineFeeds isBinary isReadOnly readOnly readWrite setFileTypeToObject)
('access' directory directoryUrl file fullName isDirectory localName name peekFor: printOn: reset size)
('read, write, position' atEnd basicNext compressFile findString: findStringFromEnd: flush next next: next:into:startingAt: next:putAll:startingAt: nextPut: nextPutAll: nextWordsInto: nextWordsPutAll: padToEndWith: peek peekLast position position: readInto:startingAt:count: readOnlyCopy setToEnd skip: truncate truncate: upTo: upToEnd verbatim:)
('primitives' primAtEnd: primClose: primCloseNoError: primFlush: primGetPosition: primOpen:writable: primRead:into:startingAt:count: primSetPosition:to: primSize: primSizeNoError: primTruncate:to: primWrite:from:startingAt:count:)
('registry' register unregister)
('finalization' actAsExecutor finalize)
('browser requests' defaultBrowserReadyWait post:target:url:ifError: post:url:ifError: primBrowserReady primURLPost:data:semaIndex: primURLPost:target:data:semaIndex: primURLRequest:semaIndex: primURLRequest:target:semaIndex: primURLRequestDestroy: primURLRequestFileHandle: primURLRequestState: requestURL:target: requestURL:target:ifError: requestURLStream: requestURLStream:ifError: waitBrowserReadyFor:ifFail:)
('dnd requests' primDropRequestFileHandle: primDropRequestFileName: requestDropStream:)
('*oSProcess-base' fileID)
!

